import React, { useState, useEffect } from 'react'
import { routes } from '../../routes'
import {
  Layout,
  Menu,
  Select,
  Divider,
  Space,
  Button,
  Modal,
  Input,
  message,
  Table,
} from 'antd'
import {
  Routes,
  Route,
  Link,
  Navigate,
  useLocation,
  useNavigate,
} from 'react-router-dom'
import styled from 'styled-components'
import {
  SettingFilled,
  PlusOutlined,
  ProjectOutlined,
  LogoutOutlined,
} from '@ant-design/icons'
import {
  getUserProjects,
  createProject,
  getProjectInfo,
  getProjectUsers,
  updateProject,
  setAdmin,
  removeAdmin,
  removeProjectUser,
} from './services'
const { Content, Sider, Header } = Layout

const SidebarDiv = styled.div`
  .sidebar {
    overflow: auto;
    height: 100vh;
    position: fixed;
    left: 0;
    top: 54px;
    bottom: 0;
  }
  .content {
    overflow: initial;
    margin-left: 200px;
  }
  .header {
    position: fixed;
    z-index: 1;
    width: 100%;
    height: 54px;
    background: #102337;
    display: flex;
    justify-content: flex-end;
    align-items: center;
    .projects {
      color: #fff;
      .ant-select-arrow {
        color: #fff;
      }
    }
    .line {
      height: 24px;
      margin: 0 15px;
      border-left: 1px solid #454545;
    }
    .setting {
      color: #fff;
      font-size: 20px;
      cursor: pointer;
    }
    .logout {
      color: #fff;
      font-size: 20px;
      cursor: pointer;
      margin-left: 20px;
    }
  }
  .headerLogo {
    height: 32px;
    margin: 16px;
    background: rgba(255, 255, 255, 0.2);
    text-align: center;
    color: #fff;
    line-height: 32px;
  }
`
const { Option } = Select

const Main = () => {
  const { pathname } = useLocation()
  const navigate = useNavigate()
  const [currentSide, setCurrentSide] = useState(null)
  const [projectsList, setProjectsList] = useState([])
  const [isAddProject, setIsAddProject] = useState(false)
  const [projectSetting, setProjectSetting] = useState(false)
  const [projectName, setProjectName] = useState(null)
  const [currentProjectName, setCurrentProjectName] = useState(null)
  const [currentProject, setCurrentProject] = useState(null)
  const [memberData, setMemberData] = useState([])
  // 表格Loading状态
  const [dataLoading, setDataLoading] = useState(false)

  const menus = routes.map(item => {
    return {
      key: item.path,
      label: <Link to={item.path}>{item.title}</Link>,
    }
  })
  const openJupyterlab = () => {
    window.open(process.env.REACT_APP_JPT, '_target')
  }
  menus.push({
    key: 'jupyterlab',
    label: <div onClick={openJupyterlab}>可视化编程</div>,
  })

  let childrenList = []
  const routesInfo = routes.map(item => {
    if (item.children.length > 0) {
      childrenList = childrenList.concat(...item.children)
    }
    return {
      path: item.path,
      exact: item?.exact,
      component: item.component,
    }
  })
  const list = routesInfo.concat(...childrenList)

  useEffect(() => {
    setCurrentSide(pathname)
  }, [pathname])

  useEffect(() => {
    getAllProjects()
  }, [sessionStorage.getItem('user_token')])

  const getAllProjects = async () => {
    const { data } = await getUserProjects()
    if (data.code === 200) {
      const list = data.data.map(item => {
        return {
          label: item.name,
          value: item.project_token,
          id: item.id,
        }
      })
      setProjectsList(list)
      setCurrentProject(list[0].value)
      sessionStorage.setItem('project_token', list[0].value)
      sessionStorage.setItem('project_id', list[0].id)
    } else {
      message.error(data.msg)
    }
  }

  const onSelectSide = val => {
    setCurrentSide(val.key)
  }

  const changeProject = (val, option) => {
    setCurrentProject(val)
    sessionStorage.setItem('project_token', val)
    sessionStorage.setItem('project_id', option.key)
  }

  const addProject = async () => {
    if (projectName) {
      const { data } = await createProject({
        name: projectName,
        user_id: sessionStorage.getItem('user_id'),
      })
      if (data.code === 200) {
        getAllProjects()
      } else {
        message.error(data.msg)
      }
      setIsAddProject(false)
      setProjectName(null)
    } else {
      message.error('项目名称不符合规范')
    }
  }

  const handleProjcet = () => {
    setIsAddProject(false)
    setProjectName(null)
  }

  const editProjectName = async () => {
    if (currentProjectName) {
      const { data } = await updateProject({
        name: currentProjectName,
        user_id: sessionStorage.getItem('user_id'),
        project_id: sessionStorage.getItem('project_id'),
      })
      if (data.code === 200) {
        getAllProjects()
      } else {
        message.error(data.msg)
      }
    } else {
      message.error('项目名称不符合规范')
    }
  }

  const openProjectSetting = async () => {
    getInfo()
    getProjectMembers()
    setProjectSetting(true)
  }

  // 获取项目详情
  const getInfo = async () => {
    const { data } = await getProjectInfo()
    if (data.code === 200) {
      setCurrentProjectName(data.data.name)
    } else {
      message.error(data.msg)
    }
  }

  // 获取成员
  const getProjectMembers = async () => {
    setDataLoading(true)
    const { data } = await getProjectUsers()
    if (data.code === 200) {
      const list = data.data.map(item => {
        return {
          username: item.username,
          role: item.role_name,
          key: item.id,
        }
      })
      setMemberData(list)
    } else {
      message.error(data.msg)
    }
    setDataLoading(false)
  }

  const addProjectModal = (
    <Modal
      title="新建项目"
      visible={isAddProject}
      onOk={addProject}
      okText="确认添加"
      onCancel={handleProjcet}
      cancelText="取消">
      <Space style={{ padding: '10px 20px 30px' }}>
        <span>项目名称：</span>
        <Input
          value={projectName}
          onChange={e => {
            setProjectName(e.target.value)
          }}
          style={{ width: '350px' }}
        />
      </Space>
    </Modal>
  )

  const setUserAdmin = async user_id => {
    const params = {
      project_id: sessionStorage.getItem('project_id'),
      user_id,
    }
    const { data } = await setAdmin(params)
    if (data.code === 200) {
      message.success('设置成功')
      getProjectMembers()
    } else {
      message.error(data.msg)
    }
  }

  const removeUserAdmin = async user_id => {
    const params = {
      project_id: sessionStorage.getItem('project_id'),
      user_id,
    }
    const { data } = await removeAdmin(params)
    if (data.code === 200) {
      message.success('移除成功')
      getProjectMembers()
    } else {
      message.error(data.msg)
    }
  }

  const removeUser = async user_id => {
    const params = {
      project_id: sessionStorage.getItem('project_id'),
      user_id,
    }
    const { data } = await removeProjectUser(params)
    if (data.code === 200) {
      message.success('移除成功')
      getProjectMembers()
    } else {
      message.error(data.msg)
    }
  }

  const memberColoums = [
    {
      title: '成员用户名',
      dataIndex: 'username',
      key: 'username',
    },
    {
      title: '角色',
      dataIndex: 'role',
      key: 'role',
    },
    {
      title: '操作',
      key: 'action',
      align: 'center',
      width: '25%',
      render: (_, record) => (
        <Space size="middle">
          {!record.role.includes('管理员') && (
            <Button
              type="link"
              onClick={() => {
                setUserAdmin(record.key)
              }}>
              添加为管理员
            </Button>
          )}
          {record.role.includes('管理员') && (
            <Button
              type="link"
              onClick={() => {
                removeUserAdmin(record.key)
              }}>
              移除管理员
            </Button>
          )}
          <Button
            onClick={() => {
              removeUser(record.key)
            }}
            type="link">
            移除
          </Button>
        </Space>
      ),
    },
  ]

  const addMembers = () => {}

  const projectSettingModal = (
    <Modal
      title="项目设置"
      visible={projectSetting}
      width={'50%'}
      footer={null}
      onCancel={() => {
        setProjectSetting(false)
      }}>
      <Space style={{ padding: '10px 20px 30px', display: 'flex' }}>
        <span>项目名称：</span>
        <Input
          style={{ width: '350px' }}
          value={currentProjectName}
          onChange={e => {
            setCurrentProjectName(e.target.value)
          }}
        />
        <Button type="primary" onClick={editProjectName}>
          确认
        </Button>
      </Space>
      <Space style={{ padding: '10px 20px 30px', display: 'flex' }}>
        <span>项目成员管理：</span>
        <Button type="link" onClick={addMembers}>
          添加成员
        </Button>
      </Space>
      <Table
        columns={memberColoums}
        dataSource={memberData}
        loading={dataLoading}
      />
    </Modal>
  )

  return (
    <SidebarDiv>
      <Layout>
        <Header className="header">
          <Select
            bordered={false}
            className="projects"
            placeholder="新建项目"
            optionLabelProp="label"
            value={currentProject}
            onChange={changeProject}
            dropdownStyle={{ minWidth: '150px' }}
            dropdownRender={menu => (
              <>
                {menu}
                <Divider
                  style={{
                    margin: '8px 0',
                  }}
                />
                <Space
                  style={{
                    padding: '0 8px',
                  }}>
                  <Button
                    type="text"
                    icon={<PlusOutlined />}
                    onClick={() => {
                      setIsAddProject(true)
                    }}>
                    添加项目
                  </Button>
                </Space>
              </>
            )}>
            {projectsList.map(item => (
              <Option key={item.id} value={item.value} label={item.label}>
                <Space>
                  <ProjectOutlined />
                  <span>{item.label}</span>
                </Space>
              </Option>
            ))}
          </Select>
          <div className="line" />
          <SettingFilled className="setting" onClick={openProjectSetting} />
          <LogoutOutlined
            className="logout"
            onClick={() => {
              sessionStorage.clear()
              navigate('/login')
            }}
          />
        </Header>
        <Layout hasSider style={{ marginTop: '54px' }}>
          <Sider className="sidebar">
            <div className="headerLogo">AI平台</div>
            <Menu
              theme="dark"
              mode="inline"
              items={menus}
              selectedKeys={currentSide}
              onSelect={onSelectSide}
            />
          </Sider>
          <Content className="content">
            <Routes>
              {list.map((route, index) => (
                <Route
                  key={index}
                  path={route.path}
                  exact={route?.exact}
                  element={<route.component />}
                />
              ))}
              <Route path="/" element={<Navigate to="datasource" replace />} />
            </Routes>
          </Content>
        </Layout>
      </Layout>
      {addProjectModal}
      {projectSettingModal}
    </SidebarDiv>
  )
}

export default Main
